#!/usr/bin/perl

# opera-hotlist2html.pl
# Copyright (C) 2001 Alan Hoyle <alan@alanhoyle.com>
#
# This program is derivitive of:  
#
# hotlist2xbel.pl
# Copyright (C) 2001 Haakon Nilsen <haakon@nilsen.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#


#
# Global Changes:  
#
# 23 March 2001:  replaced "xbel" with "html"
# 26 March 2001:  added comments and re-indented code
# 28 March 2001:  changed name to "opera-hotlist2html"
# 6 May 2001:  fixed bug in parse_url and parse_folder
# 8 May 2001:  fixed bug in parse_url and parse_folder
# 10 May 2001:  fixed bug in parse_url and parse_folder 
# 1 March 2002:  fixed bug 481261 (from sf.net), added folder list
#

$VERSION = "1.4";

sub get_next_line {
    if (!eof($in)) {
	$a = <$in>;
	$a =~ s/\n//; # Chop didn't work...
	$a =~ s/\t//;
	return $a;
    }
}

sub write_html {
    print(&get_indent ."$_[0]\n");
}

sub parse {
    &inc_indent;
    while (!eof($in)) {
	$line = get_next_line;
      SWITCH: {
	  if ($line eq "#FOLDER") {
	      &parse_folder;
	      last SWITCH;
	  }
	  if ($line eq "#URL") {
	      &parse_url;
	      last SWITCH;
	  }
      }
    }
    &dec_indent;
}

sub parse_folder {
    
#
# 23 March 2001: replaced xbel specific tags with html tags
# 5 May 2001:  fixed the "split" line so it behaves correctly.  (added ,2)
# 10 May 2001:  fixed &amp; &nbsp;, &quot;, &gt;, and &lt; to work correctly.
#
    
    my ($line, $name, $value);
    $line = get_next_line;
    while ($line ne "") {
	if ($line =~ /=/) {
	    ($name, $value) = split(/=/, $line, 2);
	    $attrib{$name} = $value;
	}
	$line = get_next_line;
    }
    $attrib{"NAME"} =~ s/\&amp\;/\&/g;
    $attrib{"NAME"} =~ s/\&gt\;/>/g;
    $attrib{"NAME"} =~ s/\&lt\;/</g;
    $attrib{"NAME"} =~ s/\&quot\;/\"/g;
    $attrib{"NAME"} =~ s/\&nbsp\;/ /g;
    
    $attrib{"NAME"} =~ s/\&/\&amp\;/g;
    $attrib{"NAME"} =~ s/\>/\&gt\;/g;
    $attrib{"NAME"} =~ s/\</\&lt\;/g;
    $attrib{"NAME"} =~ s/\"/\&quot\;/g;
    
    push @folders, $attrib{"NAME"};
    
    &write_html("<li><h3><a name=\"" .$attrib{"NAME"} ."\"></a>" .$attrib{"NAME"} ."</h3>");
    
    &write_html("<ul>");
    &inc_indent;
    while ($line ne "-") {
	$line = get_next_line;
      SWITCH: {
	  if ($line eq "#FOLDER") {
	      &parse_folder;
	      last SWITCH;
	  }
	  if ($line eq "#URL") {
	      &parse_url;
	  }
      }
    }
    &dec_indent;
    &write_html("</ul>");
}

sub parse_url {
#
# 23 March 2001: replaced xbel specific tags with html tags
# 5 May 2001:  fixed the "split" line so it behaves correctly.  (added ,2)
# 8 May 2001:  fixed &amp; and & translation to work correctly.
# 10 May 2001:  fixed &nbsp;, &quot;, &gt;, and &lt; translation to work correctly.
#
    my ($bookmark, $line, $name, $value);
    $line = get_next_line;
    while ($line ne "") {
	if ($line =~ /=/) {
	    ($name, $value) = split(/=/, $line, 2);
	    $bookmark{$name} = $value;
	}
	$line = get_next_line;
    }
    
    $bookmark{"NAME"} =~ s/\&amp\;/\&/g;
    $bookmark{"NAME"} =~ s/\&gt\;/\>/g;
    $bookmark{"NAME"} =~ s/\&lt\;/\</g;
    $bookmark{"NAME"} =~ s/\&quot\;/\"/g;
    $bookmark{"NAME"} =~ s/\&nbsp\;/ /g;
    
    $bookmark{"NAME"} =~ s/\&/\&amp\;/g;
    $bookmark{"NAME"} =~ s/\>/\&gt\;/g;
    $bookmark{"NAME"} =~ s/\</\&lt\;/g;
    $bookmark{"NAME"} =~ s/\"/\&quot\;/g;
    
    $bookmark{"URL"} =~ s/\&amp\;/\&/g;
    $bookmark{"URL"} =~ s/\&/\&amp\;/g;
    
    &write_html("<li><a href=\"" .$bookmark{"URL"} ."\">" .$bookmark{"NAME"} ."</a>");
}

sub inc_indent {
    $indent++;
}

sub dec_indent {
    $indent--;
}

sub get_indent {
    $ind = "";
    for ($i = 0; $i < $indent - 1; $i++) { $ind .= "   "; }
    return $ind;
}


if (($ARGV[0] eq "--help")||($ARGV[0] eq "-h")) {
    
# i.e. print help message
# 23 March 2001: Changed this to new, updated information
# 10 May 2001:  added more information
#
    
    print("operahotlist2html.pl $VERSION by Alan Hoyle <alan\@alanhoyle.com>\n");
    print("   derived from hotlist2xbel.pl 1.0 by Haakon Nilsen <haakon\@nilsen.com>\n");
    print("   Reads from stdin, writes to stdout.  e.g.:\n");
    print("   add a \"STRING\" after the command to personalize the output.\n");
    print("     \$ operahotlist2html.pl \"My title\" \\\n       < ~/.opera/opera.adr > ~/public_html/operahostlist.html\n");
    exit;
}  # end of help message


#
# 10 May 2001  changed $fullname to $custom_title
#

$custom_title = $ARGV[0];


$in = *STDIN;
$err = *STDERR;

$l = get_next_line;
if ($l ne "Opera Hotlist version 2.0") {
    print($err, "\nHotlist version not supported! Only 2.0 supported. No soup for you.\n");
    exit;
} 

else 

{
    $indent = 1;
    
#
# 23 March 2001: replaced xbel specific tags with html tags
# 10 May 2001:  changed behavior of "custom_title"
# 13 November 2001: added encoding type
#  1 March 2001:  added folder list
#
    
    &write_html("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/REC-html401/loose.dtd\">");
    &write_html("<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>");
    &write_html("<html>");
    &inc_indent;
    &write_html("<head><title>$custom_title</title></head>");
    &write_html("<body>");
    &inc_indent;
    &write_html("<h1>$custom_title</h1>");
    &write_html("<p><a href=\"#Folder list\">Folder list</a></p>");
    &write_html("<hr>");
    &write_html("<ul>");
    
    &parse;
    
    &write_html("</ul>");
    
    &write_html("<hr>");

    &write_html("<h4><a name=\"Folder list\"></a>Folder list:</h4>");
    
    
    &write_html("<p style=\"text-align:justify\">");
    foreach $folder (@folders){
	&write_html("[<a href=\"#$folder\">$folder</a>]");
    }
    &write_html("</p>");
    
    &write_html("<hr>");
    # 10 May 2001 added link to page
    
    &write_html("Generated with <a href=\"http://ophotlist2html.sourceforge.net/\">operahotlist.html.pl</a><p>");

    &write_html("<p>");
    &write_html("<a href=\"http://validator.w3.org/check/referer\"><img border=\"0\"");
    &write_html("src=\"http://www.w3.org/Icons/valid-html401\"");
    &write_html("alt=\"Valid HTML 4.01!\" height=\"31\" width=\"88\"></a>");
    &write_html("</p>");
    

    
    &dec_indent;
    &write_html("</body>");
    &dec_indent;
    &write_html("</html>");
}

exit;








